searchNodes=[{"type":"module","title":"TelemetryRegistry","doc":"TelemetryRegistry provides tools for the discovery and documentation of telemetry events within your applications. Telemetry Event Definitions and Declaration Users want to know what telemetry events are available in your library, what they mean, as well as what the measurements and metadata maps contain. TelemetryRegistry creates an official standard and mechanism for telemetry event declaration and definition. Who Should Document Events? Library authors in particular should provide event declarations for documentation and to simplify tracing. Of course, everyone should document their modules! Where Should I Declare Events? Events should only be declared once, usually in the module from which it originates. Event names should always be namespaced to your application or library. For example, if your application is an http client, your events should start with the name of your application, not a generic name. Do: [:tesla, :request, :stop] Don't: [:http_client, :request, :stop] Event Definition Format Events are declared using the telemetry_event module attribute. The attribute accepts an event definition which are used for producing documentation and event discovery. All definition keys are required. %{ event : [ :my_app , :event , :stop ] , description : &quot;A description of what the event is and when it is emitted&quot; , measurements : &quot;A string containing a pseudo or typespec - see examples&quot; , metadata : &quot;A string containing a pseudo or real typespec - see examples&quot; } \#{ event =&gt; [ my_app , event , stop ] , description =&gt; &lt;&lt; &quot;A description of what the event is and when it is emitted&quot; &gt;&gt; , measurements =&gt; &lt;&lt; &quot;A string containing a pseudo or typespec - see examples&quot; &gt;&gt; , metadata =&gt; &lt;&lt; &quot;A string containing a pseudo or real typespec - see examples&quot; &gt;&gt; } Elixir Elixir does not allow for declaring a custom attribute multiple times by default. We have included macros to help with this and to provide a way to include event documentation. defmodule TestElixirApp do use TelemetryRegistry telemetry_event %{ event : [ :test_elixir_app , :single , :event ] , description : &quot;emitted when this event happens&quot; , measurements : &quot;%{duration: non_neg_integer()}&quot; , metadata : &quot;%{status: status(), name: String.t()}&quot; } @moduledoc &quot;&quot;&quot; Module documentation... ## Telemetry \#{ telemetry_docs ( ) } &quot;&quot;&quot; end Add use TelemetryRegistry at the top of your module to prep your module for defining events. This handles setting up everything needed to declare events and the very helpful telemetry_event/1 macro. Event Discovery Events can be discovered by invoking discover_all , usually during application startup. The registry will walk through the application tree and aggregate all events. The events are cached, so this should only be invoked once at startup. You can view all declared events using list_events/0 . It is also possible to limit event discovery to a particular application tree by passing an application name to discover_all/1 . Distributed Tracing Event discovery is critical for supporting distributed tracing of black-box libraries used in your application. Library authors are encouraged to use telemetry events in their libraries to provide insight of internal operations to users in a vendor-agnostic manner. TelemetryRegistry provides a mechanism through spannable_events/0 for distributed tracing library authors to discover events which can be used to start and stop child spans by registering telemetry event handlers automatically at runtime with no user intervention. Library authors can then provide additional mechanisms for users to enhance spans with attributes created from telemetry event measurements and metadata.","ref":"TelemetryRegistry.html"},{"type":"function","title":"TelemetryRegistry.discover_all/0","doc":"Discover all declared telemetry events in the application it is invoked from and all child applications. This would normally be invoked during application startup.","ref":"TelemetryRegistry.html#discover_all/0"},{"type":"function","title":"TelemetryRegistry.discover_all/1","doc":"Discover all declared telemetry events in the given application and its child applications. This is typically used in libraries leveraging telemetry_registry where it would be necessary for the user to define what the root application is, e.g. in tracing bridge libraries.","ref":"TelemetryRegistry.html#discover_all/1"},{"type":"function","title":"TelemetryRegistry.docs_for/1","doc":"Generate telemetry event documentation formatted in Markdown for a given module.","ref":"TelemetryRegistry.html#docs_for/1"},{"type":"function","title":"TelemetryRegistry.list_events/0","doc":"Returns a list of all registered events. Example iex&gt; TelemetryRegistry . list_events ( ) [ { %{ description : &quot;Event description&quot; , measurements : &quot;Measurements description, metadata: &quot; Metadata description &quot; } } ]","ref":"TelemetryRegistry.html#list_events/0"},{"type":"function","title":"TelemetryRegistry.spannable_events/0","doc":"Returns a list of spannable events. Example iex&gt; TelemetryRegistry . spannable_events ( ) [ { [ :my_app , :request ] , [ :start , :stop , :exception ] } ]","ref":"TelemetryRegistry.html#spannable_events/0"},{"type":"macro","title":"TelemetryRegistry.telemetry_docs/0","doc":"Generates telemetry event documentation formatted in Markdown for use in your documentation.","ref":"TelemetryRegistry.html#telemetry_docs/0"},{"type":"macro","title":"TelemetryRegistry.telemetry_event/1","doc":"Declares a telemetry event. Accepts a telemetry event definition event_definition/0 .","ref":"TelemetryRegistry.html#telemetry_event/1"},{"type":"type","title":"TelemetryRegistry.application/0","doc":"An application to discover events from.","ref":"TelemetryRegistry.html#t:application/0"},{"type":"type","title":"TelemetryRegistry.event/0","doc":"A tuple containing the telemetry event, the module in which it was declared, and event definition meta. {:telemetry.event_name(), module(), t:event_meta()","ref":"TelemetryRegistry.html#t:event/0"},{"type":"type","title":"TelemetryRegistry.event_definition/0","doc":"An event definition is composed of an event, description, measurements description, and metadata description.","ref":"TelemetryRegistry.html#t:event_definition/0"},{"type":"type","title":"TelemetryRegistry.event_description/0","doc":"A description of what the event represents and when it is emitted.","ref":"TelemetryRegistry.html#t:event_description/0"},{"type":"type","title":"TelemetryRegistry.event_measurements/0","doc":"A string representation of the measurements emitted with the event. This should resemble a typespec but is not limited to the typespec format, i.e. you can include a comment on which unit a value is in. The objective is to inform users.","ref":"TelemetryRegistry.html#t:event_measurements/0"},{"type":"type","title":"TelemetryRegistry.event_meta/0","doc":"A map of event definition meta for an event containing the event, measurements, and metadata descriptions if the event was declared with an event definition. Otherwise, this value will be an empty map.","ref":"TelemetryRegistry.html#t:event_meta/0"},{"type":"type","title":"TelemetryRegistry.event_metadata/0","doc":"A string representation of the metadata emitted with the event. This should resemble a typespec but is not limited to the typespec format, i.e. you can include comments or example values. The objective is to inform users what is available.","ref":"TelemetryRegistry.html#t:event_metadata/0"},{"type":"type","title":"TelemetryRegistry.spannable_event/0","doc":"A list of spannable events known to the registry in the format of {event_prefix, event_suffixes} . For example, given events [:my_app, :request, :start], [:my_app, :request, :stop], [:my_app, :request, :exception] a span can be created from the :start -&gt; :stop or the :start -&gt; :exception events. These are aggregated as a spannable event {[:my_app, :request], [:start, :stop, :exception]} .","ref":"TelemetryRegistry.html#t:spannable_event/0"},{"type":"extras","title":"Overview","doc":"Telemetry Registry TelemetryRegistry is a library for Telemetry event declaration, discovery, and registration. Events are declared using the module attribute telemetry_event and include a description of the event, measurements, and metadata.","ref":"readme.html"},{"type":"extras","title":"Overview - How It Works","doc":"The Registry works by walking an application tree and examining every module for Telemetry Event definitions when invoking the discovery feature. - telemetry_event \#{ event =&gt; [ test_app , handler , start ] , description =&gt; &lt;&lt; &quot;Emitted at the start of the handler&quot; &gt;&gt; , measurements =&gt; &lt;&lt; &quot;\#{system_time =&gt; non_neg_integer()}&quot; &gt;&gt; , metadata =&gt; &lt;&lt; &quot;\#{}&quot; &gt;&gt; } . - telemetry_event \#{ event =&gt; [ test_app , handler , stop ] , description =&gt; &lt;&lt; &quot;Emitted at the end of the handler&quot; &gt;&gt; , measurements =&gt; &lt;&lt; &quot;\#{duration =&gt; non_neg_integer()}&quot; &gt;&gt; , metadata =&gt; &lt;&lt; &quot;\#{}&quot; &gt;&gt; } . - telemetry_event \#{ event =&gt; [ test_app , handler , exception ] , description =&gt; &lt;&lt; &quot;The handler raised an exception&quot; &gt;&gt; , measurements =&gt; &lt;&lt; &quot;\#{duration =&gt; non_neg_integer()}&quot; &gt;&gt; , metadata =&gt; &lt;&lt; &quot;\#{kind =&gt; atom(), reason =&gt; atom(), stacktrace =&gt; term()}&quot; &gt;&gt; } . Add the Registry to Your Application After the applications are loaded, simply run telemetry_registry : discover_all ( my_app ) . Or if you want to load all applications loaded in current VM then you can use: telemetry_registry : discover_all ( ) . Viewing Events The defined events can be accessed using list_events/0 . Events are returned as a list of three element tuples of {Event, Module, Meta} where Event is the event name, Module is the module it was discovered in, and Meta is the event definition metadata. telemetry_registry : list_events ( ) . Spannable Events Tracing spans need at least a matching start and stop event to create a child span. Optionally, a exception event can be emitted in the case of an exception being raised. spannable_events/0 returns a proplist of all matching (spannable) events that have been discovered. These are returned as a proplist with keys being the event prefix and the value being a list of the available events, e.g. [{[test_app,handler], [start,stop,exception]}] . telemetry_registry : spannable_events ( ) . %% [{[test_app,handler], [start,stop,exception]}]","ref":"readme.html#how-it-works"},{"type":"extras","title":"Overview - Elixir Users","doc":"A variety of macros to assist with event declaration and generating telemetry event documentation are available. Please refer to the HexDocs for more information. Dependency in Elixir def deps do [ { :telemetry_registry , &quot;~&gt; 0.3&quot; } ] end Copyright 2020 Bryan Naegele TelemetryRegistry source code is released under Apache License, Version 2.0.","ref":"readme.html#elixir-users"},{"type":"extras","title":"telemetry_registry","doc":"Data Types Function Index Function Details","ref":"telemetry_registry.html"},{"type":"extras","title":"telemetry_registry - Data Types","doc":"&lt;a name=&quot;type-application&quot;&gt;application()&lt;/a&gt; application ( ) = atom ( ) ### event() ### event() = { telemetry:event_name() , module(), event_meta() } ### event_definition() ### event_definition() = \#{event =&gt; telemetry:event_name() , description =&gt; event_description() , measurements =&gt; event_measurements() , metadata =&gt; event_metadata() } ### event_description() ### event_description ( ) = binary ( ) ### event_measurements() ### event_measurements ( ) = binary ( ) ### event_meta() ### event_meta() = \#{description =&gt; event_description() , measurements =&gt; event_measurements() , metadata =&gt; event_metadata() } | \#{} ### event_metadata() ### event_metadata ( ) = binary ( ) ### spannable_event() ### spannable_event() = { telemetry:prefix() , [atom()]} ## Function Index ## discover_all/0 Discover all declared telemetry events in the application it is invoked from and all child applications. discover_all/1 Discover all declared telemetry events in the given application and its child applications. list_events/0 Returns a list of all registered events. spannable_events/0 Returns a list of spannable events. ## Function Details ## ### discover_all/0 ### discover_all ( ) -&gt; ok Discover all declared telemetry events in the application it is invoked from and all child applications. This would normally be invoked during application startup. ### discover_all/1 ### discover_all(Applications:: application() | [ application() ]) -&gt; ok Discover all declared telemetry events in the given application and its child applications. This is typically used in libraries leveraging ``` telemetry_registry ``` where it would be necessary for the user to define what the root application is, e.g. in tracing bridge libraries. ### list_events/0 ### list_events() -&gt; [ event() ] Returns a list of all registered events. ### spannable_events/0 ### spannable_events() -&gt; [ spannable_event() ] Returns a list of spannable events.","ref":"telemetry_registry.html#data-types"}]